from .JsonEnum import JsonEnum

class DataType(JsonEnum):
    Unclassified = "unclassified"
    String = "string"
    Int64 = "int64"
    Double = "double"
    DateTime = "dateTime"
    DateTimeOffset = "dateTimeOffset"
    Decimal = "decimal"
    Boolean = "boolean"
    Guid = "guid"
    Json = "json"


from .Reader import Reader
from azure.storage.blob import BlockBlobService
from io import StringIO
import json
import pandas as pd


class ADLSReader(Reader):
    """
        Write dataframe/json to specified blob storage location
    """
    def __init__(self, account_name, account_key, container_name, storage_name, dataflow_name):
        self.account_name = account_name
        self.account_key = account_key
        self.container_name = container_name
        self.storage_name = storage_name
        self.dataflow_name = dataflow_name
    
    def read_df(self, locations, headers, dtypes):
        """
            Read csv files from the specified locations
        """
        dfs = []
        for blob_location in locations:
            blob_location = self.dataflow_name + blob_location.split(self.container_name)[1]
            block_blob_service = BlockBlobService(account_name=self.account_name, account_key=self.account_key)
            csv_content = block_blob_service.get_blob_to_text(self.container_name, blob_location).content
            df = pd.read_csv(StringIO(csv_content), names=headers)
            dfs.append(df)
        return pd.concat(dfs)


from .Base import Base
from .ObjectCollection import ObjectCollection
from .SchemaEntry import SchemaEntry
from .utils import String


class Annotation(Base):
    def __init__(self, schema=[]):
        self.schema = schema + [
            SchemaEntry("name", String),
            SchemaEntry("value", String)
        ]
        super().__init__(self.schema)

class AnnotationCollection(ObjectCollection):
    itemType = Annotation



from .DataObject import DataObject
from .SchemaEntry import SchemaEntry
from .FileFormatSettings import FileFormatSettings
from .MetadataObject import MetadataObjectCollection
from .utils import DateTimeOffset
from .utils import Uri


class Partition(DataObject):
    def __init__(self, schema=[]):
        self.schema = schema + [
            SchemaEntry("refreshTime", DateTimeOffset),
            SchemaEntry("location", Uri),
            SchemaEntry("fileFormatSettings", FileFormatSettings)
        ]
        super().__init__(self.schema)


class PartitionCollection(MetadataObjectCollection):
    itemType = Partition



from .Base import Base
from .JsonEnum import JsonEnum
from .utils import String
from .SchemaEntry import SchemaEntry
from .FileFormatSettings import FileFormatSettings
from collections import OrderedDict


class CsvQuoteStyle(JsonEnum):
    Csv = "QuoteStyle.Csv"
    None_ = "QuoteStyle.None"


class CsvStyle(JsonEnum):
    QuoteAlways = "CsvStyle.QuoteAlways"
    QuoteAfterDelimiter = "CsvStyle.QuoteAfterDelimiter"


class CsvFormatSettings(FileFormatSettings):
    def __init__(self, schema=[]):
        self.schema = schema + [
            SchemaEntry("columnHeaders", bool),
            SchemaEntry("delimiter", String),
            SchemaEntry("quoteStyle", CsvQuoteStyle),
            SchemaEntry("csvStyle", CsvStyle)
        ]
        super().__init__(self.schema)

    def toJson(self):
        result = OrderedDict()
        result["$type"] = "CsvFormatSettings"
        entity = super().toJson()
        for k, v in entity.items():
            result[k] = v
        return result



import abc

class Writer(metaclass=abc.ABCMeta):
        @abc.abstractmethod
        def write_df(self, location, dataframe, number_of_partition=5): ()

        @abc.abstractmethod
        def get_existing(self, location, snapshot_dir_name): ()

        @abc.abstractmethod
        def write_json(self, location, json_dict, lease_id=None): ()
